import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/auth/models/auth_industry.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import '../model/username_available_result.dart';
import 'models/check_email_result.dart';
import 'models/upload_thumbnail_result.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String checkUpdateEmail() => "/v2/auth/checkUpdateEmail";
  static String resetPasswordByEmail() => "resetPasswordByEmail";
  static String confirmResetProPasswordCode() => "confirmResetProPasswordCode";
  static String updateNewProPasswordByToken() =>
      "/v2/auth/updateNewProPasswordByToken";
  static String confirmResetUserPasswordCode() =>
      "/v2/auth/confirmResetUserPasswordCode";
  static String updateNewUserPasswordByToken() =>
      "updateNewUserPasswordByToken";
  static String setThumbnailForUserOrPro() =>
      "/v2/common/setThumbnailForUserOrPro";

  static String getServiceCatForIndustry() =>
      "v2/common/getServiceCatForIndustry";
  static String loginTerminal() => "/v2/auth/loginTerminal";
  static String loginUserOrProByUsername() =>
      "/v2/auth/loginUserOrProByUsername";
  static String refreshToken() => "/v2/auth/refreshToken";
  static String checkNewProEmail() => '/v2/auth/checkNewProEmail';
  static String confirmNewProCode() => '/v2/auth/confirmNewProCode';
  static String checkUsernameAvailable() => "v2/auth/checkUsernameAvailable";
  static String createNewPro() => 'v2/auth/createNewPro';
  static String createNewUser() => 'v2/auth/createNewUser';
  static String updateRegistrationLevelForUser() =>
      "/v2/common/updateRegistrationLevelForUser";
  static String updateRegistrationLevelForPro() =>
      "/v2/common/updateRegistrationLevelForPro";
}

abstract class MyAppApi {
  Future<NetworkResponse> checkUpdateEmail(params);
  Future<NetworkResponse> resetPasswordByEmail(params);
  Future<NetworkResponse> confirmResetProPasswordCode(params);
  Future<NetworkResponse> updateNewProPasswordByToken(params);
  Future<NetworkResponse> confirmResetUserPasswordCode(params);
  Future<NetworkResponse> updateNewUserPasswordByToken(params);

  Future<NetworkResponse> setThumbnailForUserOrPro(bytes, filePath,
      [ProgressCallback? callback]);

  Future<NetworkResponse> getUserInfo(params);
  Future<NetworkResponse> getProInfo(params);

  Future<NetworkResponse> getServiceCatForIndustry(params);

  Future<NetworkResponse> loginTerminal(params);
  Future<NetworkResponse> loginUserOrProByUsername(params);
  Future<NetworkResponse> refreshToken(refreshToken);

  Future<NetworkResponse> checkNewProEmail(params);
  Future<NetworkResponse> confirmNewProCode(params);
  Future<NetworkResponse> checkUsernameAvailable(params);
  Future<NetworkResponse> createNewPro(params);
  Future<NetworkResponse> createNewUser(params);

  Future<NetworkResponse> updateRegistrationLevelForUser(params);
  Future<NetworkResponse> updateRegistrationLevelForPro(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> loginTerminal(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.loginTerminal(), data: params);
        return NetworkResponse.fromResponse(response, value: response.data);
      },
    );
  }

  @override
  Future<NetworkResponse> loginUserOrProByUsername(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.loginUserOrProByUsername(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> resetPasswordByEmail(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.resetPasswordByEmail(), data: (params));
        return NetworkResponse.fromResponse(response,
            converter: (json) => json, prefix: "");
      },
    );
  }

  @override
  Future<NetworkResponse> confirmResetProPasswordCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.confirmResetProPasswordCode(), data: (params));
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> confirmResetUserPasswordCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).post(
            _MyAppEndpoint.confirmResetUserPasswordCode(),
            data: (params));
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateNewProPasswordByToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.updateNewProPasswordByToken(), data: (params));
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateNewUserPasswordByToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).post(
            _MyAppEndpoint.updateNewUserPasswordByToken(),
            data: (params));
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForUserOrPro(bytes, filePath,
      [ProgressCallback? callback]) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        FormData formData = FormData.fromMap({
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
          "ratio": 1,
        });
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.setThumbnailForUserOrPro(),
                data: formData, onSendProgress: callback);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UploadThumbnalResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getUserInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getUserInfo(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getProInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getProInfo(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ProInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getServiceCatForIndustry(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(_MyAppEndpoint.getServiceCatForIndustry(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => AuthIndustry.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> checkUpdateEmail(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(_MyAppEndpoint.checkUpdateEmail(), data: params);
        return NetworkResponse.fromResponse(response,
            value: UpdateInfoResult(status: response.statusCode == 200));
      },
      builder: (e) {
        if (e.response?.data != null && e.response!.data['error'] is String) {
          return NetworkResponse.fromResponse(e.response!,
              value: UpdateInfoResult(
                  status: false, msg: e.response!.data['error']));
        }
        return NetworkResponse.withErrorRequest(e);
      },
    );
  }

  @override
  Future<NetworkResponse> refreshToken(refreshToken) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(requiredToken: true, token: refreshToken)
                .post(_MyAppEndpoint.refreshToken());
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkNewProEmail(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.checkNewProEmail(), data: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> confirmNewProCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.confirmNewProCode(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkUsernameAvailable(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            _MyAppEndpoint.checkUsernameAvailable(),
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UsernameAvailableResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createNewPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.createNewPro(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> createNewUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.createNewUser(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> updateRegistrationLevelForUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.updateRegistrationLevelForUser(),
                data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateRegistrationLevelForPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.updateRegistrationLevelForPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }
}
