import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/common/models/contact_form_fields.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import 'models/platform_user_detail.dart';
import 'models/platform_user_list.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String trackOnClick() => "v2/common/trackOnClick";
  static String getPlatformsUsersFromAdmin() =>
      "v2/common/getPlatformsUsersFromAdmin";
  static String getPlatformsUserDetailsFromAdmin() =>
      "v2/common/getPlatformsUserDetailsFromAdmin";
  static String getForms() => "v2/common/getForms";
  static String fillForm() => "v2/common/fillForm";

  static String createMediaCat() => "v2/common/createMediaCat";
  static String updateMediaCat() => "v2/common/updateMediaCat";
  static String getMediaCat() => "v2/common/getMediaCat";
  static String deleteMediaCat() => "v2/common/deleteMediaCat";

  static String uploadMedia() => "v2/common/uploadMedia";
  static String updateMedia() => "v2/common/updateMedia";
  static String getMediasByFilter() => "v2/common/getMediasByFilter";
  static String getMediaByPurpose() => "v2/common/getMediaByPurpose";
  static String deleteMedia() => "v2/common/deleteMedia";
  static String getCountryCodeFromLatLng() =>
      "v2/common/getCountryCodeFromLatLng";
}

abstract class MyAppApi {
  Future<NetworkResponse> trackOnClick(params);
  Future<NetworkResponse> getPlatformsUsersFromAdmin(params);
  Future<NetworkResponse> getPlatformsUserDetailsFromAdmin(params);
  Future<NetworkResponse> getForms(params);
  Future<NetworkResponse> fillForm(params);
  Future<NetworkResponse> getMediasByFilter(params);
  Future<NetworkResponse> getMediaByPurpose(params);
  Future<NetworkResponse> getMediaCat(params);
  Future<NetworkResponse> uploadMedia(
      bytes, filePath, Map<String, dynamic> params);
  Future<NetworkResponse> createMediaCat(params);
  Future<NetworkResponse> updateMediaCat(params);
  Future<NetworkResponse> updateMedia(
      bytes, filePath, Map<String, dynamic> params);
  Future<NetworkResponse> deleteMedia(params);
  Future<NetworkResponse> deleteMediaCat(params);
  Future<NetworkResponse> getCountryCodeFromLatLng(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> getCountryCodeFromLatLng(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            _MyAppEndpoint.getCountryCodeFromLatLng(),
            queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => json['countryCode'],
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getPlatformsUsersFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getPlatformsUsersFromAdmin(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => PlatformUserList.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getPlatformsUserDetailsFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getPlatformsUserDetailsFromAdmin(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => PlatformUserDetail.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> trackOnClick(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.trackOnClick(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getForms(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(_MyAppEndpoint.getForms(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => ContactFormField.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> fillForm(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(_MyAppEndpoint.fillForm(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getMediasByFilter(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getMediasByFilter(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => CommonMedia.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getMediaByPurpose(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(_MyAppEndpoint.getMediaByPurpose(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => CommonMedia.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getMediaCat(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => CommonMedia.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> uploadMedia(
      bytes, filePath, Map<String, dynamic> params) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        Map<String, dynamic> map = {
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        };
        map.addAll(params);
        FormData formData = FormData.fromMap(map);
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.uploadMedia(), data: formData);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.createMediaCat(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.updateMediaCat(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateMedia(
      bytes, filePath, Map<String, dynamic> params) async {
    return await handleNetworkError(
      proccess: () async {
        late Map<String, dynamic> map;
        if (bytes != null) {
          String fileName = filePath.split('/').last;
          map = {
            "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
          };
        } else {
          map = {};
        }

        map.addAll(params);
        FormData formData = FormData.fromMap(map);
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(_MyAppEndpoint.updateMedia(), data: formData);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteMedia(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(_MyAppEndpoint.deleteMedia(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> deleteMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(_MyAppEndpoint.deleteMediaCat(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }
}
