import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import 'models/models.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String googlePlaceAutoComplete() => "googlePlaceAutoComplete";
  static String googlePlaceSearch() => "googlePlaceSearch";
}

abstract class MyAppApi { 

  ///Google place pass our sever
  Future<NetworkResponse> googlePlaceSearch(params);
  Future<NetworkResponse> googlePlaceSearchById(params);
  Future<NetworkResponse> googlePlaceAutoComplete(String keyword,
      {String? countryCode, List<String>? types, String? language});
}

class MyAppApiImp extends MyAppApi { 

  @override
  Future<NetworkResponse> googlePlaceSearch(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.googlePlaceSearch(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) => (json as List<dynamic>)
                .map<SeachPlaceResultModel>(
                    (i) => SeachPlaceResultModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> googlePlaceSearchById(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.googlePlaceSearch(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'result',
            converter: (json) => SeachPlaceResultDetailModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> googlePlaceAutoComplete(String keyword,
      {String? countryCode, List<String>? types, String? language}) async {
    return await handleNetworkError(
      proccess: () async {
        var queryParameters = {
          'value': keyword,
          'types': types ?? ['address']
        };
        if (countryCode != null) {
          queryParameters.addAll({
            'country': countryCode,
          });
        }
        if (language != null) {
          queryParameters.addAll({
            'language': language,
          });
        }
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.googlePlaceAutoComplete(),
                queryParameters: queryParameters);
        if (response.data['status'] == 'OK') {
          return NetworkResponse.fromResponse(response,
              prefix: 'predictions',
              converter: (json) => (json as List<dynamic>)
                  .map<AutocompletePrediction>(
                      (i) => AutocompletePrediction.fromJson(i))
                  .toList());
        } else {
          return NetworkResponse();
        }
      },
    );
  }
}
