import 'package:equatable/equatable.dart';

class BrPlaceMenuContent {
  String? id;
  String? menuTitle;
  String? pictureUrl;
  String? oobrPlaceName;
  String? qrCode;
  List<PlaceMenuMaincats>? maincats;

  BrPlaceMenuContent(
      {this.id, this.menuTitle, this.pictureUrl, this.oobrPlaceName, this.qrCode, this.maincats});

  BrPlaceMenuContent.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) id = json["id"];
    if (json["menuTitle"] is String) menuTitle = json["menuTitle"];
    if (json["pictureUrl"] is String) pictureUrl = json["pictureUrl"];
    if (json["oobrPlaceName"] is String) oobrPlaceName = json["oobrPlaceName"];
    if (json["qrCode"] is String) qrCode = json["qrCode"];
    if (json["maincats"] is List) {
      maincats = json["maincats"] == null
          ? null
          : (json["maincats"] as List).map((e) => PlaceMenuMaincats.fromJson(e)).toList();
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["menuTitle"] = menuTitle;
    data["pictureUrl"] = pictureUrl;
    data["oobrPlaceName"] = oobrPlaceName;
    data["qrCode"] = qrCode;
    if (maincats != null) data["maincats"] = maincats?.map((e) => e.toJson()).toList();
    return data;
  }
}

class PlaceMenuItems extends Equatable {
  String? id;
  int? order;
  String? name;
  String? description;
  String? pictureUrl;
  double? price;
  double? promoPrice;
  String? currency;
  int? orderOnline;
  int? quickOrderInPlace;
  String? qrCode;
  int? hasOptions;

  int quantity = 0;
  bool expand = false;
  List<PlaceMenuAllergens> allergens = [];
  List<PlaceMenuItemOption> options = [];
  List<PlaceMenuItemOption> optionsSelected = [];

  double get realPrice => promoPrice != null && promoPrice! > 0 ? promoPrice! : (price ?? 0.0);

  PlaceMenuItems(
      {this.id,
      this.order,
      this.name,
      this.pictureUrl,
      this.price,
      this.currency,
      this.orderOnline,
      this.quickOrderInPlace,
      this.qrCode,
      this.description,
      this.promoPrice,
      this.hasOptions});

  PlaceMenuItems.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) id = json["id"];
    if (json["order"] is int) order = json["order"];
    if (json["name"] is String) name = json["name"];
    if (json["description"] is String) description = json["description"];
    if (json["pictureUrl"] is String) pictureUrl = json["pictureUrl"];
    if (json["price"] is double) price = json["price"];
    if (json["promoPrice"] is double) promoPrice = json["promoPrice"];
    if (json["currency"] is String) currency = json["currency"];
    if (json["orderOnline"] is int) orderOnline = json["orderOnline"];
    if (json["quickOrderInPlace"] is int) quickOrderInPlace = json["quickOrderInPlace"];
    if (json["hasOptions"] is int) hasOptions = json["hasOptions"];
    if (json["qrCode"] is String) qrCode = json["qrCode"];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["order"] = order;
    data["name"] = name;
    data["description"] = description;
    data["pictureUrl"] = pictureUrl;
    data["price"] = price;
    data["promoPrice"] = promoPrice;
    data["currency"] = currency;
    data["orderOnline"] = orderOnline;
    data["quickOrderInPlace"] = quickOrderInPlace;
    data["hasOptions"] = hasOptions;
    data["qrCode"] = qrCode;
    return data;
  }

  @override 
  List<Object?> get props => [optionsSelected, id];
}

class PlaceMenuMaincats {
  String? id;
  int? order;
  String? catName;
  List<PlaceMenuSubcats>? subcats;

  PlaceMenuMaincats({this.id, this.order, this.catName, this.subcats});

  PlaceMenuMaincats.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) id = json["id"];
    if (json["order"] is int) order = json["order"];
    if (json["catName"] is String) catName = json["catName"];
    if (json["subcats"] is List) {
      subcats = json["subcats"] == null
          ? null
          : (json["subcats"] as List).map((e) => PlaceMenuSubcats.fromJson(e)).toList();
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["order"] = order;
    data["catName"] = catName;
    if (subcats != null) data["subcats"] = subcats?.map((e) => e.toJson()).toList();
    return data;
  }
}

class PlaceMenuSubcats {
  String? id;
  int? order;
  String? catName;
  List<PlaceMenuItems>? items;

  PlaceMenuSubcats({this.id, this.order, this.catName, this.items});

  PlaceMenuSubcats.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) id = json["id"];
    if (json["order"] is int) order = json["order"];
    if (json["catName"] is String) catName = json["catName"];
    if (json["items"] is List) {
      items = json["items"] == null
          ? null
          : (json["items"] as List).map((e) => PlaceMenuItems.fromJson(e)).toList();
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["order"] = order;
    data["catName"] = catName;
    if (items != null) data["items"] = items?.map((e) => e.toJson()).toList();
    return data;
  }
}

class PlaceMenuItemOption extends Equatable {
  String? id;
  String? currency;
  double? price;
  String? name;
  String? valueName;

  PlaceMenuItemOption({this.id, this.currency, this.price, this.name, this.valueName});

  PlaceMenuItemOption.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) id = json["id"];
    if (json["currency"] is String) currency = json["currency"];
    if (json["price"] is double) price = json["price"];
    if (json["name"] is String) name = json["name"];
    if (json["valueName"] is String) valueName = json["valueName"];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["currency"] = currency;
    data["price"] = price;
    data["name"] = name;
    data["valueName"] = valueName;
    return data;
  }

  @override 
  List<Object?> get props => [id];
}

class PlaceMenuAllergens {
  String? id;
  String? name;
  String? internationalCode;

  PlaceMenuAllergens({this.id, this.name, this.internationalCode});

  PlaceMenuAllergens.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) id = json["id"];
    if (json["name"] is String) name = json["name"];
    if (json["internationalCode"] is String) internationalCode = json["internationalCode"];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["name"] = name;
    data["internationalCode"] = internationalCode;
    return data;
  }
}
