import 'package:equatable/equatable.dart';

import '_model.dart';

class PlaylistModel extends Equatable {
  int? id;
  String? label;
  String? colorCode;
  int? repeat;
  dynamic playlistType;
  List<PlaylistItem2>? items;
  List<PlaylistSchedules>? schedules;
  List<Terminal>? terminalLinked;

  PlaylistModel({
    this.id,
    this.label,
    this.colorCode,
    this.repeat,
    this.items,
    this.schedules,
    this.playlistType,
    this.terminalLinked,
  });

  PlaylistModel.fromJson(Map<String, dynamic> json) {
    if (json["id"] is int) id = json["id"];
    if (json["label"] is String) label = json["label"];
    if (json["colorCode"] is String) colorCode = json["colorCode"];
    playlistType = json["playlistType"];
    if (json["repeat"] is int) repeat = json["repeat"];
    if (json["items"] is List) {
      items = json["items"] == null
          ? null
          : (json["items"] as List)
              .map((e) => PlaylistItem2.fromJson(e))
              .toList();
    }
    if (json["schedules"] is List) {
      schedules = json["schedules"] == null
          ? null
          : (json["schedules"] as List)
              .map((e) => PlaylistSchedules.fromJson(e))
              .toList();
    }
    if (json["terminalLinked"] is List) {
      terminalLinked = json["terminalLinked"] == null
          ? null
          : (json["terminalLinked"] as List)
              .map((e) => Terminal.fromJson(e))
              .toList();
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["label"] = label;
    data["repeat"] = repeat;
    data["playlistType"] = playlistType;
    // if (items != null) data["items"] = items?.map((e) => e.toJson()).toList();
    if (schedules != null) {
      data["schedules"] = schedules?.map((e) => e.toJson()).toList();
    }
    if (terminalLinked != null) {
      data["terminalLinked"] = terminalLinked?.map((e) => e.toJson()).toList();
    }
    return data;
  }

  @override
  List<Object?> get props => [id, playlistType, items, schedules];
}

class PlaylistSchedules extends Equatable {
  String? label;
  int? days;
  int? dateStart;
  int? dateStop;
  int? hourStart;
  int? hourStop;
  int? scheduleId;

  PlaylistSchedules({
    this.label,
    this.days,
    this.dateStart,
    this.dateStop,
    this.hourStart,
    this.hourStop,
    this.scheduleId,
  });

  List<int> get listDayOfWeek {
    return days.toString().split('').map((e) => int.parse(e)).toList();
  }

  PlaylistSchedules.fromJson(Map<String, dynamic> json) {
    if (json["label"] is String) label = json["label"];
    if (json["days"] is int) days = json["days"];
    if (json["dateStart"] is int) dateStart = json["dateStart"];
    if (json["dateStop"] is int) dateStop = json["dateStop"];
    if (json["hourStart"] is int) hourStart = json["hourStart"];
    if (json["hourStop"] is int) hourStop = json["hourStop"];
    if (json["scheduleId"] is int) scheduleId = json["scheduleId"];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["label"] = label;
    data["days"] = days;
    data["dateStart"] = dateStart;
    data["dateStop"] = dateStop;
    data["hourStart"] = hourStart;
    data["hourStop"] = hourStop;
    data["scheduleId"] = scheduleId;
    return data;
  }

  @override
  List<Object?> get props =>
      [scheduleId, days, dateStart, dateStop, hourStart, hourStop];
}

class PlaylistItem2 extends Equatable {
  int? id;
  String? label;
  dynamic value;
  int? duration;
  int? itemOrder;
  int? playlistId;
  int? audio;
  _ContentType? contentType;

  PlaylistItem2(
      {this.id,
      this.label,
      this.value,
      this.duration,
      this.itemOrder,
      this.playlistId,
      this.audio,
      this.contentType});

  PlaylistItem2.fromJson(Map<String, dynamic> json) {
    if (json["id"] is int) id = json["id"];
    if (json["label"] is String) label = json["label"];
    value = json["value"];
    if (json["duration"] is int) duration = json["duration"];
    if (json["itemOrder"] is int) itemOrder = json["itemOrder"];
    if (json["playlistId"] is int) playlistId = json["playlistId"];
    if (json["audio"] is int) audio = json["audio"];
    if (json["contentType"] is Map) {
      contentType = json["contentType"] == null
          ? null
          : _ContentType.fromJson(json["contentType"]);
    }
  }

  @override
  List<Object?> get props => [id, value, contentType, itemOrder, duration];

  // Map<String, dynamic> toJson() {
  //   final Map<String, dynamic> data = new Map<String, dynamic>();
  //   data["id"] = id;
  //   data["label"] = label;
  //   data["value"] = value;
  //   data["duration"] = duration;
  //   data["itemOrder"] = itemOrder;
  //   data["playlistId"] = playlistId;
  //   if (contentType != null) data["contentType"] = contentType?.toJson();
  //   return data;
  // }
}

class _ContentType extends Equatable {
  int? id;
  String? label;

  _ContentType();

  _ContentType.fromJson(Map<String, dynamic> json) {
    if (json["id"] is int) id = json["id"];
    if (json["label"] is String) label = json["label"];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["label"] = label;
    return data;
  }

  @override
  List<Object?> get props => [id];
}
