import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import 'models/platform_status_chat.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String getStatusChatList() => "v2/platform/getStatusChatList";
  static String getMyCurrentStatusChatForPlatformId() =>
      "v2/platform/getMyCurrentStatusChatForPlatformId";
  static String getStatusChatForPlatformId() =>
      "v2/platform/getStatusChatForPlatformId";
  static String updateStatusChatForPlatformId() =>
      "v2/platform/updateStatusChatForPlatformId";
  static String createChatGroup() => "v2/platform/createChatGroup";
  static String setThumbnailForChatGroup() =>
      "v2/platform/setThumbnailForChatGroup";
}

abstract class MyAppApi {
  Future<NetworkResponse> getStatusChatList(params);
  Future<NetworkResponse> getMyCurrentStatusChatForPlatformId(params);
  Future<NetworkResponse> getStatusChatForPlatformId(params);
  Future<NetworkResponse> updateStatusChatForPlatformId(params);
  Future<NetworkResponse> createChatGroup(params);
  Future<NetworkResponse> setThumbnailForChatGroup(groupId, bytes, filePath);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> getStatusChatList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(_MyAppEndpoint.getStatusChatList(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => (json as List)
              .map((e) => PlatformStatusChat.fromJson(e))
              .toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getMyCurrentStatusChatForPlatformId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getMyCurrentStatusChatForPlatformId(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => PlatformStatusChat.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getStatusChatForPlatformId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getStatusChatForPlatformId(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => PlatformStatusChat.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> updateStatusChatForPlatformId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(_MyAppEndpoint.updateStatusChatForPlatformId(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> createChatGroup(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(_MyAppEndpoint.createChatGroup(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForChatGroup(
      groupId, bytes, filePath) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        FormData formData = FormData.fromMap({
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
          "groupId": groupId,
        });
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(_MyAppEndpoint.setThumbnailForChatGroup(), data: formData);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }
}
