import 'dart:io';

import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';
import 'package:_private_core/_private_core.dart';
import '../auth/models/check_email_result.dart';
import '../model/info_by_username.dart'; 
import '../resources.dart';

class AppApiImp extends AppApi {

  @override
  Future<NetworkResponse> getAccountOwnershipFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.getAccountOwnershipFromPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }


  @override
  Future<NetworkResponse> checkNewPhoneFromUser(phone) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(requiredToken: false, enableErrorHandler: false)
                .get(AppEndpoint.checkNewPhoneFromUser(),
                    queryParameters: {'phone': phone});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> checkNewPhoneFromPro(phone) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(requiredToken: false, enableErrorHandler: false)
                .get(AppEndpoint.checkNewPhoneFromPro(),
                    queryParameters: {'phone': phone});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> checkExistPhoneFromUser(phone) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.checkExistPhoneFromUser(),
            queryParameters: {'phone': phone});
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> checkExistPhoneFromPro(phone) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.checkExistPhoneFromPro(),
            queryParameters: {'phone': phone});
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> addNotificationToken(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addNotificationToken(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> createOrLoginPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.createOrLoginPro(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> updateNewUserPasswordByFirebase(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.updateNewUserPasswordByFirebase(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> loginUserByPhone(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).post(
          AppEndpoint.loginUserByPhone(),
          data: params,
        );
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> loginUserByPhoneAndFirebase(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).post(
          AppEndpoint.loginUserByPhoneAndFirebase(),
          data: params,
        );
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> loginUserByEmail(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.loginUserByEmail(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> createOrLoginUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.createOrLoginUser(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> createNewFirebaseCustomToken() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(
          AppEndpoint.createNewFirebaseCustomToken(),
        );
        return NetworkResponse.fromResponse(response,
            converter: (json) => json);
      },
    );
  }

  @override
  Future<NetworkResponse> getMqttInfo() async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getMqttInfo());
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLoginMqtt.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateUser(params) async {
    return await handleNetworkError(proccess: () async {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClient(requiredToken: true, token: token)
          .put(AppEndpoint.updateUser(), data: params);
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    }, builder: (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    });
  }

  @override
  Future<NetworkResponse> updatePro(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(AppEndpoint.updatePro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: UpdateInfoResult(status: true));
      },
    );
  }

  @override
  Future<NetworkResponse> updateProFromSA(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(AppEndpoint.updateProFromSA(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateUserFromSA(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(AppEndpoint.updateUserFromSA(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> logoutPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.logoutPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> logoutUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.logoutUser(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateProPwd(
      String oldPassword, String newPassword) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(AppEndpoint.updateProPwd(),
                data: {'oldPassword': oldPassword, 'newPassword': newPassword});
        return NetworkResponse.fromResponse(response,
            value: UpdateInfoResult(status: response.statusCode == 200));
      },
      builder: (e) {
        if (e.response?.data != null && e.response!.data['error'] is String) {
          return NetworkResponse.fromResponse(e.response!,
              value: UpdateInfoResult(
                  status: false, msg: e.response!.data['error']));
        }
        return NetworkResponse.withErrorRequest(e);
      },
    );
  }

  @override
  Future<NetworkResponse> updateUserPwd(
      String oldPassword, String newPassword) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(AppEndpoint.updateUserPwd(),
                data: {'oldPassword': oldPassword, 'newPassword': newPassword});
        return NetworkResponse.fromResponse(response,
            value: UpdateInfoResult(status: response.statusCode == 200));
      },
      builder: (e) {
        if (e.response?.data != null && e.response!.data['error'] is String) {
          return NetworkResponse.fromResponse(e.response!,
              value: UpdateInfoResult(
                  status: false, msg: e.response!.data['error']));
        }
        return NetworkResponse.withErrorRequest(e);
      },
    );
  }
 

  @override
  Future<NetworkResponse> confirmUpdateUserEmailCode(email, code) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.confirmUpdateUserEmailCode(),
                data: {'email': email, 'code': code});
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmEmailCodeResponse.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateUserEmail(token) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateUserEmail(), data: {'token': token});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }


  @override
  Future<NetworkResponse> confirmUpdateProEmailCode(email, code) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.confirmUpdateProEmailCode(),
                data: {'email': email, 'code': code});
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmEmailCodeResponse.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateProEmail(token) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateProEmail(), data: {'token': token});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> checkExistEmailFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(AppEndpoint.checkExistEmailFromPro(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkIfEmailExist(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.checkIfEmailExist(), data: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkNewEmailUserEmailConfirm(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.checkNewEmailUserEmailConfirm(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ConfirmOtp.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkEmailPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.checkEmailPro(), data: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => CheckEmailResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getCategoriesList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getCategoriesList(),
            queryParameters: {'catLanguageCode': null});
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => (json['maincats'] as List)
              .map((e) => MainCat.fromJson(e))
              .toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> loginPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.loginPro(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  } 

  @override
  Future<NetworkResponse> loginProByPhoneAndFirebase(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.loginProByPhoneAndFirebase(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> loginProCrmByQrCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.loginProCrmByQrCode(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> getInfoByUsername(username) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getInfoByUsername(),
            queryParameters: {'username': username});
        return NetworkResponse.fromResponse(response,
            converter: (json) => InfoByUsername.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkLoginProByEmailOTP(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.checkLoginProByEmailOTP(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> loginProByEmailOTPConfirm(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.loginProByEmailOTPConfirm(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TokenResponseLogin.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> getAssetsForProFromUniversalDashboard(
      proUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getAssetsForProFromUniversalDashboard(),
            queryParameters: {'proUniqueId': proUniqueId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => OrgProInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getDateFormatList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(AppEndpoint.getDateFormatList());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<DateFormatModel>((i) => DateFormatModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getTimeFormatList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(AppEndpoint.getTimeFormatList());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<TimeFormatModel>((i) => TimeFormatModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getIndustriesList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getIndustriesList(),
            queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<IndustryModel>((i) => IndustryModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getCurrencies() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getCurrencies(),
            queryParameters: {'available': 1});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<CurrencyAvailable>((i) => CurrencyAvailable.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getAllTerminalFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAllTerminalFromAdmin(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => Terminal.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getCurrentTerminalsOfPlaylistFromPro(
      params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2_getCurrentTerminalsOfPlaylistFromPro(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => Terminal.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getVideoConfigFromPlayerVimeo(vimeoId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getVideoConfigFromPlayerVimeo(),
            queryParameters: {'videoId': vimeoId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => VimeoConfig.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateTerminalFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateTerminalFromPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateProduct(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateProduct(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> takeMissionFromSeller(missionId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.takeMissionFromSeller(),
                data: {'missionId': missionId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getProductCategories() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProductCategories(),
                queryParameters: {'display': 1});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProductCategory>((i) => ProductCategory.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getProductsFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProductsFromPro(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProductModel>((i) => ProductModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getProductOfProFromSA(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProductOfProFromSA(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProductModel>((i) => ProductModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getProductOfUserFromSA(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProductOfUserFromSA(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProductModel>((i) => ProductModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getStatusList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getStatusList());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProductStatus>((i) => ProductStatus.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getAllPublicEventsFromSearch(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getAllPublicEventsFromSearch(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEvent>((i) => PublishEvent.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getAllPublicEventsAroundMe(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getAllPublicEventsAroundMe(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEvent>((i) => PublishEvent.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPublicEventsListFromUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getPublicEventsListFromUser(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEvent>((i) => PublishEvent.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getSuggestionEventsFromUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getSuggestionEventsFromUser(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEvent>((i) => PublishEvent.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPublicEventsCategories(languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getPublicEventsCategories(),
                queryParameters: {'languageCode': languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEventCategory>(
                    (i) => PublishEventCategory.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getFavoritePublicEventsCategoriesFromUser(
      languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getFavoritePublicEventsCategoriesFromUser(),
                queryParameters: {'languageCode': languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEventCategory>(
                    (i) => PublishEventCategory.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getLikedPublicEventsFromUser() async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getLikedPublicEventsFromUser());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PublishEvent>((i) => PublishEvent.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getTicketListFromPublicEvent(publicEventId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getTicketListFromPublicEvent(),
                queryParameters: {'publicEventId': publicEventId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<TicketInfoModel>((i) => TicketInfoModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> addUserLikeToPublicEvent(publicEventId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.addUserLikeToPublicEvent(),
                data: {'publicEventId': publicEventId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setFavoriteCatForUser(catId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.setFavoriteCatForUser(), data: {'catId': catId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> removeFavoriteCatForUser(catId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .delete(AppEndpoint.removeFavoriteCatForUser(),
                data: {'catId': catId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> addSeenSuggestionEventFromUser(
      publicEventId, utcModifier) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.addSeenSuggestionEventFromUser(), data: {
          'publicEventId': publicEventId,
          'utcModifier': utcModifier
        });
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> bookFreeTicketForPublicEvents(
      arckipelTicketId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.bookFreeTicketForPublicEvents(),
                data: {'arckipelTicketId': arckipelTicketId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getAllowedFeaturesForOowooApp() async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getAllowedFeaturesForOowooApp());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<AllowedFeatureModel>(
                    (i) => AllowedFeatureModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getDashboardConfigFromPro() async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.v2_getDashboardConfigFromPro(),
                queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => DashboardConfig.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createCustomerCardStripe(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.createCustomerCardStripe(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => PaymentCardStripe.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteCustomerCardStripe(cardId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .delete(AppEndpoint.deleteCustomerCardStripe(),
                data: {'cardId': cardId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getCustomerCardsStripe(page, pageSize) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getCustomerCardsStripe(),
                queryParameters: {'page': page, 'page_size': pageSize});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PaymentCardStripe>((i) => PaymentCardStripe.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> setCustomerCardDefaultStripe(cardId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .put(AppEndpoint.setCustomerCardDefaultStripe(),
                data: {'cardId': cardId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> becomeSeller(confirmed) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.becomeSeller(), data: {'confirmed': confirmed});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateImageForBecomeSeller(fileType, path,
      [ProgressCallback? callback]) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = path.split('/').last;
        FormData formData = FormData.fromMap({
          "fileType": fileType,
          "fileName": MultipartFile.fromBytes(File(path).readAsBytesSync(),
              filename: fileName),
        });
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.updateImageForBecomeSeller(),
                data: formData, onSendProgress: callback);
        return NetworkResponse.fromResponse(response,
            value: response.data['values']['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> quitSellerProgram(temporary) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.quitSellerProgram(),
                data: {'temporary': temporary});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> recoverySellerProgram() async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.recoverySellerProgram());
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getTncOowooApp() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getTncOowooApp(),
            queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['text']);
      },
    );
  }

  @override
  Future<NetworkResponse> getTncOowooSeller() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getTncOowooSeller(),
            queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['text']);
      },
    );
  }

  @override
  Future<NetworkResponse> birthsOnThisDay(date) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get('https://byabbe.se/on-this-day/$date/births.json');
        return NetworkResponse.fromResponse(response,
            prefix: 'births',
            converter: (json) => (json as List<dynamic>)
                .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> deathsOnThisDay(date) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get('https://byabbe.se/on-this-day/$date/deaths.json');
        return NetworkResponse.fromResponse(response,
            prefix: 'deaths',
            converter: (json) => (json as List<dynamic>)
                .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> eventsOnThisDay(date) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get('https://byabbe.se/on-this-day/$date/events.json');
        return NetworkResponse.fromResponse(response,
            prefix: 'events',
            converter: (json) => (json as List<dynamic>)
                .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> pexelsSearch(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true,
                isAuthorizationCustom: true,
                token: PEXEL_API_KEY)
            .get('https://api.pexels.com/v1/search', queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'photos',
            converter: (json) => (json as List<dynamic>)
                .map<PexelPhotoModel>((i) => PexelPhotoModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> weatherForecast(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.weatherapi.com/v1/forecast.json',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'forecast',
            converter: (json) => WeatherForecast.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> weatherHistory(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.weatherapi.com/v1/history.json',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'forecast',
            converter: (json) => WeatherForecast.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getMissionsFromSeller(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getMissionsFromSeller(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<SellerMissionModel>((i) => SellerMissionModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getMissionTypes(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getMissionTypes(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<MissionTypeModel>((i) => MissionTypeModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOoBRPlaceInfoById(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOoBRPlaceInfoById(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => BrPlaceModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getOoBRPlacesAroundMe(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOoBRPlacesAroundMe(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOoBRPlacesByFilterValues(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOoBRPlacesByFilterValues(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getFavoriteOobrPlaces() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getFavoriteOobrPlaces());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> updateFilterValueForOobr(params) async {
    return await handleNetworkError(proccess: () async {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.updateFilterValueForOobr(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['updateFilterValueForOobr']);
    }, builder: (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['error']);
      }
    });
  }

  @override
  Future<NetworkResponse> getPromotedVideosForBar(oobrPlaceId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPromotedVideosForBar(),
                queryParameters: {'oobrPlaceId': oobrPlaceId});
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List<dynamic>).map<String>((i) => i).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPromotedVideosForOobrPlaces(lat, lng) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPromotedVideosForOobrPlaces(),
                queryParameters: {'lat': lat, 'lng': lng});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPromotedVideoModel>(
                    (i) => BrPromotedVideoModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPromotedVideosForOoh(lat, lng) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPromotedVideosForOoh(),
                queryParameters: {'lat': lat, 'lng': lng});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPromotedVideoModel>(
                    (i) => BrPromotedVideoModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOoBRCategories(languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOoBRCategories(),
                queryParameters: {'languageCode': languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceCateModel>((i) => BrPlaceCateModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOoBRFilters(languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOoBRFilters(),
                queryParameters: {'languageCode': languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceFilterModel>((i) => BrPlaceFilterModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> checkBarGooglePlaceIdByContributor(
      googlePlaceId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.checkBarGooglePlaceIdByContributor(),
                queryParameters: {'googlePlaceId': googlePlaceId});
        return NetworkResponse.fromResponse(response, value: '');
      },
      builder: (e) {
        if (e.response != null &&
            e.response!.statusCode == 400 &&
            e.response!.data != null) {
          return NetworkResponse.fromResponse(e.response!,
              value: e.response!.data['oobrPlaceId']);
        }
        return NetworkResponse.withErrorRequest(e);
      },
    );
  }

  @override
  Future<NetworkResponse> createNewOobrPlaceByContributor(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createNewOobrPlaceByContributor(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> addOobrPlaceToFavorites(oobrPlaceId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addOobrPlaceToFavorites(),
                data: {'oobrPlaceId': oobrPlaceId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getOobrPlaceSchedules(oobrPlaceId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOobrPlaceSchedules(),
                queryParameters: {'oobrPlaceId': oobrPlaceId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceScheduleModel>(
                    (i) => BrPlaceScheduleModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOobrPlaceMenuList(
      oobrPlaceId, languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOobrPlaceMenuList(), queryParameters: {
          'oobrPlaceId': oobrPlaceId,
          'languageCode': languageCode
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<BrPlaceMenu>((i) => BrPlaceMenu.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOobrPlaceMenuContent(
      oobrPlaceId, languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOobrPlaceMenuContent(), queryParameters: {
          'oobrPlaceId': oobrPlaceId,
          'languageCode': languageCode
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => BrPlaceMenuContent.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getOptionsForItem(itemId, languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOptionsForItem(), queryParameters: {
          'itemId': itemId,
          'languageCode': languageCode
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PlaceMenuItemOption>(
                    (i) => PlaceMenuItemOption.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getAllergensForItem(itemId, languageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAllergensForItem(), queryParameters: {
          'itemId': itemId,
          'languageCode': languageCode
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PlaceMenuAllergens>((i) => PlaceMenuAllergens.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getContactsList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getContactsList(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ContactModel>((i) => ContactModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> addUserInContactsList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addUserInContactsList(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ContactModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> addUserOrProToContactListByFirebaseId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addUserOrProToContactListByFirebaseId(),
                data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ContactModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createFirebaseDynamicLink(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.createFirebaseDynamicLink(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json);
      },
    );
  }

  @override
  Future<NetworkResponse> deleteContact(id) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.deleteContact(), data: {'id': id});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateCommentForContact(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateCommentForContact(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getExistingUserByPhone(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getExistingUserByPhone(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ContactModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createOrderForOobrPlace(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createOrderForOobrPlace(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<UserModel>((i) => UserModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getAnythingFromQr(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAnythingFromQr(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => QrDataModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getDetailOobrInvitations(oobrEventId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getDetailOobrInvitations(),
                queryParameters: {'oobrEventId': oobrEventId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => BrPlaceInviteModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> inviteContactsToOobrPlace(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.inviteContactsToOobrPlace(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> answerOobrInvitation(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.answerOobrInvitation(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getEmergencyInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getEmergencyInfos(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => EmergencyResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> sendEmergencyRequest(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.sendEmergencyRequest(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> createSharedPcEvent(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createSharedPcEvent(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['id']);
      },
    );
  }

  @override
  Future<NetworkResponse> getPcEventFromUser(pcEventId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPcEventFromUser(),
                queryParameters: {'pcEventId': pcEventId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => PeEventModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForPcEvent(path, pcEventId) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = path.split('/').last;
        FormData data = FormData.fromMap({
          'pcEventId': pcEventId,
          "fileName": MultipartFile.fromBytes(File(path).readAsBytesSync(),
              filename: fileName),
        });
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setThumbnailForPcEvent(), data: data);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> shareCalendarToUsers(params) =>
      throw UnimplementedError();

  @override
  Future<NetworkResponse> whatsNext(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.whatsNext(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => WhatNextModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> generateToken(channelName, expiredTime) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.generateVideoCallToken(),
                data: {'channel': channelName});
        return NetworkResponse.fromResponse(response,
            converter: (json) => AgoraToken.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getNumberOfSalesFromSellers(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getNumberOfSalesFromSellers(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => NumberOfSaleModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> setBankAccountFromUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setBankAccountFromUser(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => BankAccountModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateBankInfosFromSeller(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateBankInfosFromSeller(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => BankAccountModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getAccountTypes() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAccountTypes());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => BankAccountTypeModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getBankInfosFromSeller() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getBankInfosFromSeller());
        return NetworkResponse.fromResponse(response,
            converter: (json) => BankAccountModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> addUserOrProToTrackingPeople(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addUserOrProToTrackingPeople(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => TrackingInfoModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getTrackingInfosForMeeting(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getTrackingInfosForMeeting(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => TrackingInfoModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> updateTrackingInfosForMeeting(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateTrackingInfosForMeeting(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => TrackingInfoModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getLearnMoreTrackingVideo() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getLearnMoreTrackingVideo(),
            queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => json);
      },
    );
  }

  @override
  Future<NetworkResponse> getReminderTypes() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getReminderTypes(),
                queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => ReminderType.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOowooMoveProperties(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOowooMoveProperties(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => MoveModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getOowooVideoByTypeId(typeId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOowooVideoByTypeId(), queryParameters: {
          'typeId': typeId,
          'languageCode': appPrefs.languageCode
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => VideoUrl.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getCalmByCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getCalmByCat(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<CalmModel>((i) => CalmModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getCalmCategories() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getCalmCategories(),
                queryParameters: {'languageCode': appPrefs.languageCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<CalmCate>((i) => CalmCate.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getSubscribersForCalmId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getSubscribersForCalmId(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json['subscribers'] as List<dynamic>)
                .map<CalmSubscriber>((i) => CalmSubscriber.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> subscribeToCalmId(calmId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.subscribeToCalmId(), data: {'calmId': calmId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> testSendNotificationToPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.testSendNotificationToPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> testSendNotificationToUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.testSendNotificationToUser(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> notifVideoCallProFromUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.notifVideoCallProFromUser(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> notifVideoCallUserFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.notifVideoCallUserFromPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getReviewsFromPro(proId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getReviewsFromPro(),
                queryParameters: {'proId': proId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ReviewModel>((i) => ReviewModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> cancelTicketByUser(
      ticketUniqueId, status, comment) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.cancelTicketByUser(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> confirmTicketByUser(
      ticketUniqueId, status, comment) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.confirmTicketByUser(), data: {
          'ticketUniqueId': ticketUniqueId,
          'status': status,
          'comment': comment
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteEventsFromUser(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.deleteEventsFromUser(), data: {
          'events': [ticketUniqueId]
        });
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getQueueInfoFromUser(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getQueueInfoFromUser(), queryParameters: {
          'catLanguageCode': appPrefs.languageCode,
          'ticketUniqueId': ticketUniqueId
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => EventQueueModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> ticketUserCallBack(ticketUniqueId, duration) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.ticketUserCallBack(),
                data: {'ticketUniqueId': ticketUniqueId, 'duration': duration});
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getUserInfoByFirebaseId(firebaseId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getUserInfoByFirebaseId(),
                queryParameters: {'firebaseId': firebaseId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getProInfoByFirebaseId(firebaseId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProInfoByFirebaseId(),
                queryParameters: {'firebaseId': firebaseId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => ProsModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> addToFavourite(proId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addToFavourite(), data: {'proId': proId});
        return NetworkResponse.fromResponse(response, value: true);
      },
    );
  }

  @override
  Future<NetworkResponse> getFavouriteForUser() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getFavouriteForUser());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProsModel>((i) => ProsModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> removeFromFavourite(proId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.removeFromFavourite(), data: {'proId': proId});
        return NetworkResponse.fromResponse(response, value: true);
      },
    );
  }

  @override
  Future<NetworkResponse> getPcEventFromPro(pcEventId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPcEventFromPro(),
                queryParameters: {'pcEventId': pcEventId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => PeEventModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteCodeFromLog(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.deleteCodeFromLog(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> createCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createCode(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => DigicodeCodeModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateLiveCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateLiveCode(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> shareCodeDigicode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.shareCodeDigicode(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> deleteLiveCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.deleteLiveCode(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getCodesFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getCodesFromPro(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => DigicodeCodeModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getDigicodesFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getDigicodesFromPro(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => DigicodeModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getDigicodeById(id) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getDigicodeById(), queryParameters: {'id': id});
        return NetworkResponse.fromResponse(response,
            converter: (json) => DigicodeModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkDigicodeBySerialAndToken(
      productSerial, authcode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                options: BaseOptions(headers: {'auth-code': authcode}),
                requiredToken: true,
                token: await appPrefs.getNormalToken())
            .get(AppEndpoint.checkDigicodeBySerialAndToken(),
                queryParameters: {'productSerial': productSerial});
        return NetworkResponse.fromResponse(response,
            converter: (json) => DigicodeCheck.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> linkDigicodeToPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.linkDigicodeToPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> linkDigicodeToProV2(params) async {
    return await handleNetworkError(proccess: () async {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkDigicodeToProV2(), data: params);
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    }, builder: (e) {
      if (e.response?.data != null && e.response!.data['error'] is String) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    });
  }

  @override
  Future<NetworkResponse> linkTerminalToProV2(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.linkTerminalToProV2(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getDigicodesLogs(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getDigicodesLogs(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => DigicodeLogModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPlacesFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPlacesFromPro());
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PlaceModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPlatformsList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .get(AppEndpoint.getPlatformsList(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PlatformModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPlatformsListDoubleAuth(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPlatformsListDoubleAuth(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PlatformModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> activateDoubleAuthForPlatform(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.activateDoubleAuthForPlatform(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> createPlace(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createPlace(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => PlaceModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updatePlace(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updatePlace(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> deletePlace(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.deletePlace(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getPlaceCatByLang(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPlaceCatByLang(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PlaceCatModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> unlockDigicode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.unlockDigicode(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> checkDigicodeOnline(digicodeId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.checkDigicodeOnline(),
                queryParameters: {'digicodeId': digicodeId});
        return NetworkResponse.fromResponse(response,
            prefix: 'status', converter: (json) => json);
      },
    );
  }

  @override
  Future<NetworkResponse> setLanguagesForPro(proLanguageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setLanguagesForPro(),
                data: {'proLanguageCode': proLanguageCode});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setMainLanguageForPro(proLanguageCode) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setMainLanguageForPro(),
                data: {'proLanguageCode': proLanguageCode});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getAllServicesOfProFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAllServicesOfProFromPro(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => ProServiceModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> callNextTicket(serviceId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.callNextTicket(), data: {'serviceId': serviceId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> cancelTicketByPro(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.cancelTicketByPro(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> canceledByProAfterCallBack(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.canceledByProAfterCallBack(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => EventInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> confirmTicketByPro(
      ticketUniqueId, status, comment) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.confirmTicketByPro(), data: {
          'ticketUniqueId': ticketUniqueId,
          'confirm': status,
          'comment': comment
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => EventInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createTicketByPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createTicketByPro(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getAllServicesOfProFromUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAllServicesOfProFromUser(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ProServicesBusiness.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getAllServicesAroundMe(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAllServicesAroundMe(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => ServicesAroundMeModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> createTicketByUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createTicketByUser(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> declareUserAsAbsentFromPro(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.declareUserAsAbsentFromPro(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            converter: (json) => EventInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteEventsFromPro(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.deleteEventsFromPro(), data: {
          'events': [ticketUniqueId]
        });
        return NetworkResponse.fromResponse(response,
            converter: (json) => EventInfoModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getListTimeFramesByServiceForDay(
      dateToday, serviceId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getListTimeFramesByServiceForDay(),
                queryParameters: {
              'dateToday': dateToday,
              'serviceId': serviceId
            });
        return NetworkResponse.fromResponse(response,
            converter: (json) => ListTimeFrameModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getServiceAvailabilityForMonth(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getServiceAvailabilityForMonth(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => ServiceAvailabilityModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> startTicket(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.startTicket(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> stopTicket(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.stopTicket(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> ticketUserNotResponse(ticketUniqueId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.ticketUserNotResponse(),
                data: {'ticketUniqueId': ticketUniqueId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateDelay(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.updateDelay(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => ServiceDelayModel.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getLoginSessions(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getLoginSessions(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => LoginSession.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getLoginSessionsById(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getLoginSessionsById(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => LoginSession.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createNonExistingRandomCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.createNonExistingRandomCode(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json);
      },
    );
  }

  @override
  Future<NetworkResponse> revokeLoginSession(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.revokeLoginSession(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> allowPendingLoginOnPlatformByQrCode(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.allowPendingLoginOnPlatformByQrCode(),
                data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> allowPendingLoginOnPlatformById(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.allowPendingLoginOnPlatformById(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> activatePourDevice(deviceId) async {
    return await handleNetworkError(proccess: () async {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.activatePourDevice(), data: {'deviceId': deviceId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    }, builder: (e) {
      if (e.response?.statusCode == 400) {
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['error']);
      }
      return NetworkResponse.withErrorRequest(e);
    });
  }

  @override
  Future<NetworkResponse> createPurchaseByCardOnFileStripe(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createPurchaseByCardOnFileStripe(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PaymentInfoModel>((i) => PaymentInfoModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getBgVideo(countryCode) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getBgVideo(),
                queryParameters: {'countryCode': countryCode});
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['videoUrl']);
      },
    );
  }

  @override
  Future<NetworkResponse> getCurrencyAvailableForToken(tokenType) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getCurrencyAvailableForToken(),
            queryParameters: {'tokenType': tokenType});
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List<dynamic>).map<String>((i) => i).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getCurrentTokenValue(tokenType,
      {currency = 'EUR'}) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getCurrentTokenValue(),
            queryParameters: {'tokenType': tokenType, 'currency': currency});
        return NetworkResponse.fromResponse(response,
            converter: (json) => double.parse(json.toString()));
      },
    );
  }

  @override
  Future<NetworkResponse> getHistoryFromUser() async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getHistoryFromUser());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<History>((i) => History.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getPmbDevicesAroundMe(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.getPmbDevicesAroundMe(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PmbDevice>((i) => PmbDevice.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getWalletInfo() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getWalletInfo());
        return NetworkResponse.fromResponse(response,
            converter: (json) => WalletInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> ipAPI() async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(requiredToken: false, baseUrl: 'http://ip-api.com/')
                .get('json');
        return NetworkResponse.fromResponse(response,
            converter: (json) => IpInfo.fromJson(json), prefix: '');
      },
    );
  }

  @override
  Future<NetworkResponse> scanDeviceQr(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.scanDeviceQr(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ScanQr.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> checkAndAddNewContact(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .post(AppEndpoint.checkAndAddNewContact(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_addItemToPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_addItemToPlaylist(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => PlaylistItemModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> v2_createPlaylistFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_createPlaylistFromPro(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => PlaylistModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> v2_deletePlaylistFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .delete(AppEndpoint.v2_deletePlaylistFromPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getItemContentFromPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.v2_getItemContentFromPlaylist(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PlaylistItemModel>((i) => PlaylistItemModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getPlaylistFromPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .get(AppEndpoint.v2_getPlaylistFromPro(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PlaylistModel>((i) => PlaylistModel.fromJson(i))
                .toList());
      },
    );
  }

  // @override
  // Future<NetworkResponse> v2_getPlaylistsFromTerminal(params) async {
  //   bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
  //   if (isDisconnect) return NetworkResponse.withDisconnect();
  //   try {
  //     final token = await appPrefs.getNormalToken();
  //     Response response = await AppClients(requiredToken: true, token: token)
  //         .get(AppEndpoint.v2_getPlaylistsFromTerminal(),
  //             queryParameters:params);
  //     return NetworkResponse.fromResponse(response,
  //         converter: (json) => (json as List<dynamic>)
  //             .map<PlaylistModel>((i) => PlaylistModel.fromJson(i))
  //             .toList());
  //   } , builder: (e)  {
  //     return NetworkResponse.withErrorRequest(e);
  //   } catch (e) {
  //     return NetworkResponse.withErrorConvert(e);
  //   }
  // }

  @override
  Future<NetworkResponse> v2_removeItemFromPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(requiredToken: true, token: token)
            .delete(AppEndpoint.v2_removeItemFromPlaylist(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_linkPlaylistToTerminalForPro(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_linkPlaylistToTerminalForPro(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_addScheduleForPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_addScheduleForPlaylist(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_updateScheduleForPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.v2_updateScheduleForPlaylist(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_removeScheduleForPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.v2_removeScheduleForPlaylist(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_updateOrderItemsFromPlaylist(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.v2_updateOrderItemsFromPlaylist(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getProblemTypesList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2_getProblemTypesList(), queryParameters: params);

        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProblemType>((i) => ProblemType.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getFaqGroupsFromPlatformId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getFaqGroupsFromPlatformId(),
                queryParameters: params);

        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<FaqGroup>((i) => FaqGroup.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> clickOnFaq(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.clickOnFaq(), queryParameters: params);

        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProblemType>((i) => ProblemType.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getProblemTypesForGroupId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProblemTypesForGroupId(),
                queryParameters: params);

        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<ProblemType>((i) => ProblemType.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getFaqForProblemType(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getFaqForProblemType(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<FaqItem>((i) => FaqItem.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getMyTicketsFromCustomer(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2_getMyTicketsFromCustomer(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          prefix: '',
          converter: (json) => ListTicket.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> v2_postTicketByCustomer(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_postTicketByCustomer(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketDetail.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getMyTicketsFromAgent(ticketStatusId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2_getMyTicketsFromAgent(), queryParameters: {
          'languageCode': appPrefs.languageCode,
          'ticketStatusId': ticketStatusId,
        });
        return NetworkResponse.fromResponse(
          response,
          prefix: '',
          converter: (json) => ListTicket.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getAgentList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2_getAgentList(), queryParameters: {
          'languageCode': appPrefs.languageCode,
        });
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => AgentModel.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> v2_answerToTicketFromAgent(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_answerToTicketFromAgent(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> v2_answerToTicketFromCustomer(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_answerToTicketFromCustomer(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> v2_assignTicketToAgents(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.v2_assignTicketToAgents(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getAnswersOfTicket(ticketId) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAnswersOfTicket(), queryParameters: {
          'ticketId': ticketId,
        });
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => TicketAnswer.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getLinksListFromMainLink(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false).get(
            AppEndpoint.getLinksListFromMainLink(),
            queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => MainLink.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> addLocationForUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addLocationForUser(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => LocationModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createPurchaseByPaymentMethodStripe(params) {
    // TODO: implement createPurchaseByPaymentMethodStripe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> createPurchaseByTokenStripe(params) {
    // TODO: implement createPurchaseByTokenStripe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getAllEventsFromEventsWebsite(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getAllEventsFromEventsWebsite(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PublishEvent.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getCategoriesFromEventsWebsite(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getCategoriesFromEventsWebsite(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => PublishEventCategory.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getHashtagsList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getHashtagsList(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => HashTagModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getLocationFromUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getLocationFromUser(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => LocationModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getMaincatList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getMaincatList(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => MainCatModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getResultsByCatFromEventsWebsite(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getResultsByCatFromEventsWebsite(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PublishEvent.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getResultsFromFilters(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getResultsFromFilters(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => ProsModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getResultsFromSearch(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getResultsFromSearch(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => ProsModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getSpecList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getSpecList(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => SpecCatModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getSubcatList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getSubcatList(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => SubCatModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> removeLocationForUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.removeLocationForUser(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setLocationDefault(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.setLocationDefault(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateLocationForUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateLocationForUser(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  } 

  @override
  Future<NetworkResponse> getVideoPlaylistsFromTerminal(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getVideoPlaylistsFromTerminal(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getServicesListFromTerminal() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getServicesListFromTerminal());
        return NetworkResponse.fromResponse(response,
            converter: (json) => QueueOowooModel.listFromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getNextTicketAvailableForQueueFromTerminal(id) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(
          AppEndpoint.getNextTicketAvailableForQueueFromTerminal(),
          queryParameters: {'serviceId': id},
        );
        return NetworkResponse.fromResponse(response,
            converter: (json) => TicketModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> cancelNextTicketAvailableForQueueFromTerminal(
      id) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(
          AppEndpoint.cancelNextTicketAvailableForQueueFromTerminal(),
          data: {'ticketUniqueId': id},
        );
        return NetworkResponse.fromResponse(
          response,
          value: response.data,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> v2_getPlaylistsFromTerminal(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2_getPlaylistsFromTerminal());
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => PlaylistModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getListOfProsFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getListOfProsFromAdmin(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => ProInfo.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getProInfosFromSA(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getProInfosFromSA(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ProInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getUserInfosFromSA(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getUserInfosFromSA(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => UserModel.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getListOfRoles(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getListOfRoles(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => RolePro.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getListOfUsersFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getListOfUsersFromAdmin(),
                queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => UserModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> setProRole(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.setProRole(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> v2GetProductCategories(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2GetProductCategories(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => ProductCategory.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> v2getProductRefByCatId(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.v2getProductRefByCatId(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => ProductReference.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> setKisblePublicThumbnailForPro(
      bytes, path, proId) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = path.split('/').last;
        FormData data = FormData.fromMap({
          'proId': proId,
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        });
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setKisblePublicThumbnailForPro(), data: data);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForProFromSA(bytes, path, proId) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = path.split('/').last;
        FormData data = FormData.fromMap({
          'proId': proId,
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        });
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setThumbnailForProFromSA(), data: data);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> setThumbnailForUserFromSA(bytes, path, proId) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = path.split('/').last;
        FormData data = FormData.fromMap({
          'proId': proId,
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        });
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.setThumbnailForUserFromSA(), data: data);
        return NetworkResponse.fromResponse(response,
            converter: (json) => json['fileName']);
      },
    );
  }

  @override
  Future<NetworkResponse> getPaymentMethodList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPaymentMethodList(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => PaymentMethod.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getPaymentStatusList(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getPaymentStatusList(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => PaymentStatus.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getOrdersFromProOrUser(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOrdersFromProOrUser(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => PaymentOrder.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getOrdersFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getOrdersFromAdmin(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => PaymentOrder.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> updateOrderDetailsByAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateOrderDetailsByAdmin(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbSearchMulti(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/search/multi',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbSearchPerson(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/search/person',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMoviePerson.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbSearchMovie(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/search/movie',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetail(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/movie/$id',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => TheMovieDetail.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbMovieReview(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/movie/$id/reviews',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieReview.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbTrending(
      mediaType, timeWindow, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/trending/$mediaType/$timeWindow',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbDiscoverMovie(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/discover/movie',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbGenres(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/genre/movie/list',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'genres',
            converter: (json) => (json as List)
                .map((e) => TheMovieDetailGenres.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailCredits(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/movie/$id/credits',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => TheMovieCredit.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailImages(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/movie/$id/images',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '', converter: (json) => TheMovieImage.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailVideos(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/movie/$id/videos',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieVideo.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbPersonDetail(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/person/$id',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '',
            converter: (json) => TheMoviePersonDetail.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbPersonCredits(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/person/$id/movie_credits',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '',
            converter: (json) => TheMoviePersonCredit.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbPersonDetailImages(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/person/$id/images',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '',
            converter: (json) => TheMoviePersonImage.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbMovieDetailSimilar(id, params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/movie/$id/similar',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) =>
                (json as List).map((e) => TheMovieModel.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbSearchKeyword(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/search/keyword',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: 'results',
            converter: (json) => (json as List)
                .map((e) => TheMovieKeyword.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> themoviedbConfigLanguage(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
          requiredToken: false,
          enableErrorHandler: false,
        ).get('https://api.themoviedb.org/3/configuration/languages',
            queryParameters: params);
        return NetworkResponse.fromResponse(response,
            prefix: '',
            converter: (json) => (json as List)
                .map((e) => TheMovieLanguage.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> createMovieCollection(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.createMovieCollection(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TheMovieCollection.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> addMovieBookmarksToCollection(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addMovieBookmarksToCollection(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> removeMovieFromCollection(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .delete(AppEndpoint.removeMovieFromCollection(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> addMovieInArckipelDb(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .post(AppEndpoint.addMovieInArckipelDb(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => TmmAddToArckipelResult.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getMyBookmarks(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getMyBookmarks(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => TheMovieBookmark.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getMyMovieCollections(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .get(AppEndpoint.getMyMovieCollections(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List)
                .map((e) => TheMovieCollection.fromJson(e))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> updateMovieCollection(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateMovieCollection(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateMyMovieInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                requiredToken: true, token: await appPrefs.getNormalToken())
            .put(AppEndpoint.updateMyMovieInfos(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getHomesAroundMe(params) {
    // TODO: implement getHomesAroundMe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> updateProPhone(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.updateProPhone(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> updateUserPhone(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(requiredToken: false)
            .post(AppEndpoint.updateUserPhone(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }
}
