import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/model/info_by_username.dart';
import '../auth/models/check_email_result.dart';
import '../model/username_available_result.dart';
import '../resources.dart';

class UpdateInfoResult {
  bool status;
  String? msg;

  UpdateInfoResult({required this.status, this.msg});
}

class AuthRepo {
  AuthRepo._();

  static AuthRepo? _instance;

  factory AuthRepo([AppApi? api]) {
    _instance ??= AuthRepo._();
    _instance!._api = api ?? AppApiImp();
    return _instance!;
  }

  late AppApi _api;

  // ///Phone on MOBILE
  // Future<void> verifyPhoneNumber({
  //   required String phoneNumber,
  //   required PhoneVerificationCompleted verificationCompleted,
  //   required PhoneVerificationFailed verificationFailed,
  //   required PhoneCodeSent codeSent,
  //   required PhoneCodeAutoRetrievalTimeout codeAutoRetrievalTimeout,
  // }) {
  //   return FirebaseAuth.instance.verifyPhoneNumber(
  //       phoneNumber: phoneNumber,
  //       verificationCompleted: verificationCompleted,
  //       verificationFailed: verificationFailed,
  //       codeSent: codeSent,
  //       timeout: const Duration(seconds: 90),
  //       codeAutoRetrievalTimeout: codeAutoRetrievalTimeout);
  // }

  // ///Phone on WEB
  // Future<ConfirmationResult?> signInWithPhoneNumber(String phone) async {
  //   try {
  //     ConfirmationResult confirmationResult =
  //         await FirebaseAuth.instance.signInWithPhoneNumber(phone);
  //     return confirmationResult;
  //   } catch (e) {
  //     appDebugPrint(e);
  //   }
  // }

  // Future<UserCredential> confirmationResult(
  //         ConfirmationResult confirmationResult, String code) =>
  //     confirmationResult.confirm(code);

  // ///=============
  // Future<UserCredential> signInWithCredential(AuthCredential credential) =>
  //     FirebaseAuth.instance.signInWithCredential(credential);

  // Future<UserCredential> signInWithEmailPassword(
  //         String email, String password) =>
  //     FirebaseAuth.instance
  //         .signInWithEmailAndPassword(email: email, password: password);

  /// API
  Future<bool> checkNewPhoneFromPro(phone) async {
    NetworkResponse response = await _api.checkNewPhoneFromPro(phone);
    return response.data ?? false;
  }

  Future<bool> checkNewPhoneFromUser(phone) async {
    NetworkResponse response = await _api.checkNewPhoneFromUser(phone);
    return response.data ?? false;
  }

  Future<UserModel?> checkExistPhoneFromUser(phone) async {
    NetworkResponse response = await _api.checkExistPhoneFromUser(phone);
    return response.data;
  }

  Future<bool> updateNewUserPasswordByFirebase(params) async {
    NetworkResponse response =
        await _api.updateNewUserPasswordByFirebase(params);
    return response.data ?? false;
  }

  Future<NetworkResponse> loginUserByPhone(params) async {
    return await _api.loginUserByPhone(params);
  }

  Future<TokenResponseLogin?> loginUserByEmail(params) async {
    NetworkResponse response = await _api.loginUserByEmail(params);
    return response.data;
  }

  Future<TokenResponseLogin?> createOrLoginPro(params) async {
    NetworkResponse response = await _api.createOrLoginPro(params);
    return response.data;
  }

  Future<String?> createNewFirebaseCustomToken() async {
    NetworkResponse response = await _api.createNewFirebaseCustomToken();
    return response.data;
  }

  Future<TokenResponseLoginMqtt?> getMqttInfo() async {
    NetworkResponse response = await _api.getMqttInfo();
    return response.data;
  }

  Future<UpdateInfoResult> updatePro(params) async {
    NetworkResponse response = await _api.updatePro(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateUser(params) async {
    NetworkResponse response = await _api.updateUser(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateProPwd(
      {required String oldPassword, required String newPassword}) async {
    NetworkResponse response =
        await _api.updateProPwd(oldPassword, newPassword);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateUserPwd(
      {required String oldPassword, required String newPassword}) async {
    NetworkResponse response =
        await _api.updateUserPwd(oldPassword, newPassword);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<bool> addNotificationToken([fcmToken, provider]) async {
    try {
      if (fcmToken == null) {
        fcmToken = await coreOptions.getTokenFCM?.call();
        provider = 'fcm';
      }
      Map<String, dynamic> params = {
        'fcmToken': fcmToken,
        "provider": provider
      };
      NetworkResponse response = await _api.addNotificationToken(params);
      return response.data ?? false;
    } catch (e) {
      appDebugPrint('addNotificationToken: $e');
      return false;
    }
  }

  Future<bool> getAccountOwnershipFromPro(params) async {
    NetworkResponse response = await _api.getAccountOwnershipFromPro(params);
    return response.data ?? false;
  }

  Future<bool> logoutPro([fcmToken]) async {
    NetworkResponse response = await _api.logoutPro({'fcmToken': fcmToken});
    return response.data ?? false;
  }

  Future<bool> logoutUser([fcmToken]) async {
    NetworkResponse response = await _api.logoutUser({'fcmToken': fcmToken});
    return response.data ?? false;
  }

  Future<NetworkResponse> confirmUpdateUserEmailCode(email, code) async {
    return await _api.confirmUpdateUserEmailCode(email, code);
  }

  Future<bool> updateUserEmail(token) async {
    NetworkResponse response = await _api.updateUserEmail(token);
    return response.data ?? false;
  }

  Future<NetworkResponse> confirmUpdateProEmailCode(email, code) async {
    return await _api.confirmUpdateProEmailCode(email, code);
  }

  Future<bool> updateProEmail(token) async {
    NetworkResponse response = await _api.updateProEmail(token);
    return response.data ?? false;
  }

  Future<NetworkResponse> loginPro(params) async {
    return await _api.loginPro(params);
  }

  Future<NetworkResponse> loginUserByPhoneAndFirebase(params) async {
    return await _api.loginUserByPhoneAndFirebase(params);
  }

  Future<NetworkResponse> loginProByPhoneAndFirebase(params) async {
    return await _api.loginProByPhoneAndFirebase(params);
  }

  Future<TokenResponseLogin?> loginProCrmByQrCode(params) async {
    NetworkResponse response = await _api.loginProCrmByQrCode(params);
    return response.data;
  }

  Future<TokenResponseLogin?> createOrLoginUser(params) async {
    NetworkResponse response = await _api.createOrLoginUser(params);
    return response.data;
  }

  Future<CheckEmailResult?> checkEmailPro(email) async {
    NetworkResponse response = await _api.checkEmailPro({
      "email": email,
      'languageCode': appPrefs.languageCode,
    });
    return response.data;
  }

  Future<CheckEmailResult?> checkIfEmailExist(email) async {
    NetworkResponse response = await _api.checkIfEmailExist({
      "email": email,
      'allowPrivacyTypeId': 1,
      'platformId': valuePlatformByConfig(),
      'languageCode': appPrefs.languageCode
    });
    return response.data;
  }

  Future<ConfirmOtp?> checkNewEmailUserEmailConfirm({email, code}) async {
    NetworkResponse response = await _api.checkNewEmailUserEmailConfirm({
      "email": email,
      "code": code,
    });
    return response.data;
  }

  Future<bool> checkLoginProByEmailOTP(email) async {
    NetworkResponse response = await _api.checkLoginProByEmailOTP({
      "email": email,
      'platformId': valuePlatformByConfig(),
      'languageCode': appPrefs.languageCode
    });
    return response.data ?? false;
  }

  Future loginProByEmailOTPConfirm(params) async {
    return await _api.loginProByEmailOTPConfirm(params);
  }


  Future<List<MainCat>> getCategoriesList() async {
    NetworkResponse response = await _api.getCategoriesList();
    return response.data ?? [];
  }

  Future<CheckEmailResult?> checkExistEmailFromPro(email) async {
    NetworkResponse response = await _api.checkExistEmailFromPro({
      "email": email,
      'languageCode': appPrefs.languageCode,
    });
    return response.data;
  }

  Future<InfoByUsername?> getInfoByUsername(username) async {
    NetworkResponse response = await _api.getInfoByUsername(username);
    return response.data;
  }

  Future<UserModel?> getUserInfoByFirebaseId(firebaseId) async {
    NetworkResponse response = await _api.getUserInfoByFirebaseId(firebaseId);
    return response.data;
  }

  Future<ProsModel?> getProInfoByFirebaseId(firebaseId) async {
    NetworkResponse response = await _api.getProInfoByFirebaseId(firebaseId);
    return response.data;
  }

  Future<OrgProInfo?> getAssetsForProFromUniversalDashboard(proUniqueId) async {
    NetworkResponse response =
        await _api.getAssetsForProFromUniversalDashboard(proUniqueId);
    return response.data;
  }

  Future<String?> setKisblePublicThumbnailForPro(bytes, path, proId) async {
    NetworkResponse response =
        await _api.setKisblePublicThumbnailForPro(bytes, path, proId);
    return response.data;
  }

  Future<String?> setThumbnailForProFromSA(bytes, path, proId) async {
    NetworkResponse response =
        await _api.setThumbnailForProFromSA(bytes, path, proId);
    return response.data;
  }

  Future<String?> setThumbnailForUserFromSA(bytes, path, proId) async {
    NetworkResponse response =
        await _api.setThumbnailForUserFromSA(bytes, path, proId);
    return response.data;
  }

  Future<UpdateInfoResult> updateProFromSA(params) async {
    NetworkResponse response = await _api.updateProFromSA(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<UpdateInfoResult> updateUserFromSA(params) async {
    NetworkResponse response = await _api.updateUserFromSA(params);
    return response.data ?? UpdateInfoResult(status: false);
  }

  Future<bool> updateProPhone(params) async {
    NetworkResponse response = await _api.updateProEmail(params);
    return response.data ?? false;
  }

  Future<bool> updateUserPhone(token) async {
    NetworkResponse response = await _api.updateUserPhone(token);
    return response.data ?? false;
  }

  Future<UserModel?> checkExistPhoneFromPro(phone) async {
    NetworkResponse response = await _api.checkExistPhoneFromPro(phone);
    return response.data;
  }
}
