import 'dart:collection';
import 'dart:convert';
import 'package:flutter/foundation.dart';

import 'package:_private_core/_private_core.dart';
import 'package:_private_core/theme/option_network.dart';
import 'package:dio/dio.dart';

import 'network_resources/resources.dart';

_parseAndDecode(String response) {
  return jsonDecode(response);
}

parseJson(String text) {
  return compute(_parseAndDecode, text);
}

const String methodGet = "GET";
const String methodPost = "POST";
const String methodPut = "PUT";
const String methodDelete = "DELETE";

var appHeaders = {
  "Access-Control-Allow-Origin": "*",
  "Access-Control-Allow-Credentials": true,
  "Access-Control-Allow-Headers":
      "Origin,Content-Type,X-Amz-Date,Authorization,X-Api-Key,X-Amz-Security-Token,locale",
  "Access-Control-Allow-Methods": "POST, OPTIONS"
};

Map<String, dynamic> appMapParms([Map? value]) {
  Map<String, dynamic> params = {
    'platformUuid': coreOptions.platformUuid,
    'filterPlatformUuid': coreOptions.platformUuid,
    'apiKey': coreOptions.apiKey,
    'languageCode': appPrefs.languageCode,
  };
  if (value != null && value.isNotEmpty) {
    params.addAll(HashMap.from(value));
  }
  return params;
}

Map<String, dynamic> platformUuidParms([Map? value]) {
  Map<String, dynamic> params = {
    'filterPlatformUuid': coreOptions.platformUuid,
    'platformUuid': coreOptions.platformUuid,
    'languageCode': appPrefs.languageCode,
  };
  if (value != null && value.isNotEmpty) {
    params.addAll(HashMap.from(value));
  }
  return params;
}

FormData platformUuidFormData([FormData? value]) {
  FormData formData = value ?? FormData();
  if (!formData.fields.any((e) => e.key == "filterPlatformUuid")) {
    formData.fields
        .add(MapEntry("filterPlatformUuid", coreOptions.platformUuid));
  }
  if (!formData.fields.any((e) => e.key == "platformUuid")) {
    formData.fields.add(MapEntry("platformUuid", coreOptions.platformUuid));
  }
  if (!formData.fields.any((e) => e.key == "languageCode")) {
    formData.fields.add(MapEntry("languageCode", appPrefs.languageCode));
  }
  return formData;
}

PNetworkOptionsImpl get networkOptions =>
    coreOptions.networkOptions as PNetworkOptionsImpl;

String get appBaseUrl => networkOptions.baseUrl;
String get appBaseUrlAsset => networkOptions.baseUrlAsset;
String? get appBaseUrlGeotracking => networkOptions.baseUrlGeotracking;
String? get appMqttUrl => networkOptions.mqttUrl;
int? get appMqttPort => networkOptions.mqttPort;
Function(DioException)? get errorInterceptor => networkOptions.errorInterceptor;

class PNetworkOptionsImpl extends PNetworkOptions {
  final Function(DioException)? errorInterceptor;

  final String? responsePrefixData;
  final String? Function(Response? response)? responsePrefixMsg;
  final bool Function(NetworkResponse response)? responseIsSuccess;

  //loging setting
  final bool loggingEnable;
  final bool loggingrequestHeader;
  final bool loggingrequestBody;
  final bool loggingresponseBody;
  final bool loggingresponseHeader;
  final bool loggingerror;
  final bool loggingcompact;
  final int loggingmaxWidth;

  PNetworkOptionsImpl({
    required super.baseUrl,
    required super.baseUrlAsset,
    super.baseUrlGeotracking,
    super.mqttUrl,
    super.mqttPort,
    this.errorInterceptor,
    this.loggingEnable = true,
    this.loggingrequestHeader = false,
    this.loggingrequestBody = true,
    this.loggingresponseBody = true,
    this.loggingresponseHeader = false,
    this.loggingerror = true,
    this.loggingcompact = true,
    this.loggingmaxWidth = 120,
    this.responsePrefixData,
    this.responsePrefixMsg,
    this.responseIsSuccess,
  });

  @override
  String appImageCorrectUrl(String url, {base}) {
    if (url.trim().indexOf('http') != 0) {
      if ((base ?? baseUrlAsset ?? '').endsWith('/') && url.startsWith('/')) {
        return (base ?? baseUrlAsset ?? '') + url.substring(1);
      }
      return (base ?? baseUrlAsset ?? '') + url;
    }
    return url;
  }
}
